import 'dart:io';
import 'package:http/http.dart' as http;
import 'api_docs.dart';

class WebServices {
  static const String _basUrl = 'https://staging.api.mycover.ai/v1';
  static const String _initialiseSdkUrl = '$_basUrl/sdk/initialize';
  static const String _buySdkUrl = '$_basUrl/sdk/buy';
  static const String _uploadUrl = '$_basUrl/sdk/upload-file';
  static const String _ussdProviderUrl = '$_basUrl/sdk/ussd-providers';
  static const String _verifyPaymentUrl = '$_basUrl/sdk/verify-transaction';

  static const String productId = 'a72c4e3c-e868-4782-bb35-df6e3344ae6c';
  static const String userId = 'olakunle@mycovergenius.com';

  static makeClaims({
    required token,
    required policyId,
    required incidentDate,
    required claimDescription,
    required repairEstimate,
    required claimType,
    required claimOwner,
    required claimOwnerFirstName,
    required claimOwnerLastName,
    required claimOwnerEmail,
    required claimOwnerPhone,
    required timeStamp,
    required address,
    required lon,
    required lat,
    required File interior,
    required File dashboard,
    required File frontSide,
    required File backSide,
    required File leftSide,
    required File rightSide,
    required File chassisNumber,
    required videoUrl,
    required incidentType,
    required lostType,
    required claimInvoiceUrl,
  }) async {
    Map<String, String> headers = {
      "Accept": "application/json",
      "Authorization": "Bearer " + token
    };

    var uri = Uri.parse(ApiScheme.makeClaimsUrl);

    var request = http.MultipartRequest("POST", uri);

    request.files
        .add(await http.MultipartFile.fromPath('interior', interior.path));
    request.files
        .add(await http.MultipartFile.fromPath('front_side', frontSide.path));
    request.files
        .add(await http.MultipartFile.fromPath('back_side', backSide.path));
    request.files
        .add(await http.MultipartFile.fromPath('left_side', leftSide.path));
    request.files
        .add(await http.MultipartFile.fromPath('right_side', rightSide.path));
    request.files
        .add(await http.MultipartFile.fromPath('dashboard', dashboard.path));
    request.files.add(
        await http.MultipartFile.fromPath('chasis_number', chassisNumber.path));

    request.headers.addAll(headers);

    request.fields["policy_id"] = policyId;
    request.fields["incident_date"] = incidentDate;
    request.fields["claim_description"] = claimDescription;
    request.fields["repair_estimate"] = repairEstimate;
    request.fields["claim_type"] = claimType == 'Auto' ? 'vehicle' : 'gadget';
    request.fields["claim_owner"] = claimOwner;
    request.fields["claim_invoice_url"] = claimInvoiceUrl;
    request.fields["claim_owner_first_name"] = claimOwnerFirstName;
    request.fields["claim_owner_last_name"] = claimOwnerLastName;
    request.fields["claim_owner_email"] = claimOwnerEmail;
    request.fields["claim_owner_phone_number"] = claimOwnerPhone;
    request.fields["incident_type"] = incidentType;
    request.fields["loss_type"] = lostType;
    request.fields['timestamp'] = timeStamp;
    request.fields['address'] = address;
    request.fields['video_url'] = videoUrl;
    request.fields['longitude'] = lon.toString();
    request.fields['latitude'] = lat.toString();

    var response = await request.send();
    return response;
  }

  static inspection(
      {token,
      customerId,
      policyId,
      providerId,
      distributorId,
      timeStamp,
      required File interior,
      required File dashboard,
      required File frontSide,
      required File backSide,
      required File leftSide,
      required File rightSide,
      required File chassisNumber,
      address,
      lon,
      lat,
      inspectionType,
      videoUrl}) async {
    Map<String, String> headers = {
      "Accept": "application/json",
      "Authorization": "Bearer " + token
    };

    var uri = Uri.parse(ApiScheme.submitInspectionUrl);

    var request = http.MultipartRequest("POST", uri);

    request.files
        .add(await http.MultipartFile.fromPath('interior', interior.path));
    request.files
        .add(await http.MultipartFile.fromPath('front_side', frontSide.path));
    request.files
        .add(await http.MultipartFile.fromPath('back_side', backSide.path));
    request.files
        .add(await http.MultipartFile.fromPath('left_side', leftSide.path));
    request.files
        .add(await http.MultipartFile.fromPath('right_side', rightSide.path));
    request.files
        .add(await http.MultipartFile.fromPath('dashboard', dashboard.path));
    request.files.add(
        await http.MultipartFile.fromPath('chasis_number', chassisNumber.path));

    request.headers.addAll(headers);

    request.fields['customer_id'] = customerId;
    request.fields['policy_id'] = policyId;
    request.fields['provider_id'] = providerId;
    request.fields['distributor_id'] = distributorId;
    request.fields['inspection_type'] = inspectionType;
    request.fields['timestamp'] = timeStamp;
    request.fields['address'] = address ?? 'hehhe';
    request.fields['video_url'] = videoUrl;
    request.fields['longitude'] = lon.toString();
    request.fields['latitude'] = lat.toString();

    var response = await request.send();
    print(response.statusCode);
    print(response.reasonPhrase);

    return response;
  }

  static uploadFile(context, businessId, File image) async {
    Map<String, String> headers = {
      "Accept": "application/json",
      "x-api-id": "$businessId",
    };
    print(_uploadUrl);
    var uri = Uri.parse(_uploadUrl);

    var request = http.MultipartRequest("POST", uri);

    request.files.add(await http.MultipartFile.fromPath('file', image.path));

    request.fields['fileType'] = 'image';
    request.headers.addAll(headers);

    var response = await request.send();

    return response;
  }

  static getListData(String url) async {
    return await ApiScheme.initialiseGetRequest(url: '$_basUrl$url');
  }

  static getUssdProvider() async {
    return await ApiScheme.initialiseGetRequest(url: _ussdProviderUrl);
  }

  static buyProduct({
    required String userId,
    required String businessId,
    String? productId,
    payload,
    paymentChannel,
  }) async {
    var data = {
      "payload": payload,
      "payment_channel": paymentChannel,
    };

    return await ApiScheme.initialisePostRequest(
        url: _buySdkUrl, data: data, apiKey: businessId);
  }
}
